(* ------------------------------------------------------------------------------
  Curves                  : 1.0
  Copyright  1986-2018   : Copyright Adirondack Software & Graphics
  Last Modification       : 08-10-2018
  Compiler                : Delphi Tokyo
  Target Platform         : VCL Win32/Win64
  Operating System        : Windows 10
  ImageEn                 : 8.0.1
  This file is copyright  W W Miller, 1986-2018
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  ------------------------------------------------------------------------------ *)

unit uCurves;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.ExtDlgs, hyieutils, iexBitmaps, hyiedefs,
  iesettings, iexLayers, iexRulers, iexColorCurve, ieview, imageenview,
  Vcl.WinXCtrls;

type
  TFormCurves = class(TForm)
    ImageEnView1: TImageEnView;
    pnlTool: TPanel;
    labPreset: TLabel;
    labChannel: TLabel;
    Label1: TLabel;
    Open1: TButton;
    Reset1: TButton;
    Presets1: TComboBox;
    Add1: TButton;
    Channel1: TComboBox;
    Save1: TButton;
    IEColorCurve1: TIEColorCurve;
    ResetAll1: TButton;
    OpenPictureDialog1: TOpenPictureDialog;
    SavePictureDialog1: TSavePictureDialog;
    Panel1: TPanel;
    Ok1: TButton;
    Cancel1: TButton;
    Apply1: TButton;
    Guidelines1: TToggleSwitch;
    CursorSight1: TToggleSwitch;
    CircularPoints1: TToggleSwitch;
    SmallFilledPoints1: TToggleSwitch;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure Channel1Change(Sender: TObject);
    procedure Reset1Click(Sender: TObject);
    procedure ResetAll1Click(Sender: TObject);
    procedure Guidelines1Click(Sender: TObject);
    procedure CursorSight1Click(Sender: TObject);
    procedure CircularPoints1Click(Sender: TObject);
    procedure SmallFilledPoints1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Presets1Change(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Add1Click(Sender: TObject);
    procedure Apply1Click(Sender: TObject);
  private
    procedure GetPresetCurves;
    { Private declarations }
  public
    { Public declarations }
    Applied: boolean;
  end;

var
  FormCurves: TFormCurves;

implementation

{$R *.dfm}

Uses Unit1;

procedure TFormCurves.GetPresetCurves();
var
  iCurvesFolder: string;
  iFiles: TStringList;
  iSearchRec: TSearchRec;
begin
  iCurvesFolder := ExtractFileDir(Application.ExeName) + '\Curves\';
  if not DirectoryExists(iCurvesFolder) then
    exit;

  Presets1.Items.Clear;
  iFiles := TStringList.Create;
  try
    if FindFirst(iCurvesFolder + '*.acv', faAnyFile, iSearchRec) = 0 then
    begin
      repeat
        iFiles.Add(ChangeFileExt(iSearchRec.Name, ''));
      until FindNext(iSearchRec) <> 0;
      FindClose(iSearchRec);
    end;

    iFiles.Sort();
    Presets1.Items.Assign(iFiles);
  finally
    iFiles.Free;
  end;
end;

procedure TFormCurves.Add1Click(Sender: TObject);
var
  iCurveName: String;
  acvFilename: String;
begin
  iCurveName := InputBox('Curve Name', 'Specify a curve name', '');
  if iCurveName = '' then
    exit;
  acvFilename := ExtractFileDir(Application.ExeName) + '\Curves\' +
    iCurveName + '.acv';
  IEColorCurve1.SaveCurveToFile(acvFilename);
  if Presets1.Items.IndexOf(iCurveName) < 0 then
    Presets1.Items.Add(iCurveName);
  Presets1.ItemIndex := Presets1.Items.IndexOf(iCurveName);
end;

procedure TFormCurves.Open1Click(Sender: TObject);
begin
  if OpenPictureDialog1.Execute then
  begin
    ImageEnView1.IO.LoadFromFile(OpenPictureDialog1.FileName);
    IEColorCurve1.AssignSource(ImageEnView1.IEBitmap);
  end;
end;

procedure TFormCurves.ResetAll1Click(Sender: TObject);
begin
  IEColorCurve1.ResetAllCurves();
  Presets1.ItemIndex := -1;
end;

procedure TFormCurves.Reset1Click(Sender: TObject);
begin
  IEColorCurve1.ResetChannelCurve(IEColorCurve1.Channel);
  Presets1.ItemIndex := -1;
end;

procedure TFormCurves.Save1Click(Sender: TObject);
begin
  if IEColorCurve1.CurveChanged then
    IEColorCurve1.ApplyCurve(ImageEnView1.IEBitmap);
  IEColorCurve1.CurveChanged := False;

  if SavePictureDialog1.Execute then
    ImageEnView1.IEBitmap.Write(SavePictureDialog1.FileName);
end;

procedure TFormCurves.Apply1Click(Sender: TObject);
var
  iImageEnView: TImageEnView;
begin
  if Assigned(Form1.PageControl1.ActivePage) then
  begin
    iImageEnView := TImageEnView(Form1.PageControl1.ActivePage.Controls[0]);
    if Assigned(iImageEnView) then
    begin
      iImageEnView.Proc.SaveUndo('ColorCurve ' +
        IntToStr(iImageEnView.Proc.UndoCount + 1));
      Form1.UpdateUndo;
      FormCurves.IEColorCurve1.ApplyCurve(iImageEnView.IEBitmap);
      iImageEnView.Update();
      iImageEnView.IEBitmap.Modified := True;
      Applied := True;
    end;
  end;

end;

procedure TFormCurves.Channel1Change(Sender: TObject);
begin
  IEColorCurve1.Channel := TIECurveChannel(Channel1.ItemIndex);
end;

procedure TFormCurves.Presets1Change(Sender: TObject);
var
  iFilename: String;
begin
  if Presets1.ItemIndex <> -1 then
  begin
    iFilename := ExtractFileDir(Application.ExeName) + '\Curves\' +
      Presets1.Items[Presets1.ItemIndex] + '.acv';
    IEColorCurve1.LoadCurveFromFile(iFilename);
  end;
end;

procedure TFormCurves.CircularPoints1Click(Sender: TObject);
begin
  if CircularPoints1.State = tssOn then
    IEColorCurve1.PointShape := iepsCircle
  else
    IEColorCurve1.PointShape := iepsSquare;
  IEColorCurve1.Update();
end;

procedure TFormCurves.CursorSight1Click(Sender: TObject);
begin
  if CursorSight1.State = tssOn then
    IEColorCurve1.DisplayOptions := IEColorCurve1.DisplayOptions +
      [coCursorSight]
  else
    IEColorCurve1.DisplayOptions := IEColorCurve1.DisplayOptions -
      [coCursorSight];
end;

procedure TFormCurves.Guidelines1Click(Sender: TObject);
begin
  if Guidelines1.State = tssOn then
    IEColorCurve1.DisplayOptions := IEColorCurve1.DisplayOptions +
      [coGuidelines]
  else
    IEColorCurve1.DisplayOptions := IEColorCurve1.DisplayOptions -
      [coGuidelines];
end;

procedure TFormCurves.SmallFilledPoints1Click(Sender: TObject);
begin
  if SmallFilledPoints1.State = tssOn then
  begin
    IEColorCurve1.PointSize := 4;
    IEColorCurve1.PointFill := clRed;
  end
  else
  begin
    IEColorCurve1.PointSize := 8;
    IEColorCurve1.PointFill := clNone;
  end;
  IEColorCurve1.Update();
end;

procedure TFormCurves.FormCreate(Sender: TObject);
begin
  Channel1.ItemIndex := 0;
  IEColorCurve1.PointFill := clNone;
  IEColorCurve1.DisplayOptions := IEColorCurve1.DisplayOptions +
    [coCursorSight];
  IEColorCurve1.DisplayOptions := IEColorCurve1.DisplayOptions + [coGuidelines];
  IEColorCurve1.PointShape := iepsCircle;
  IEColorCurve1.PointSize := 8;
  if coCursorSight in IEColorCurve1.DisplayOptions then
    CursorSight1.State := tssOn
  else
    CursorSight1.State := tssOff;
  if coGuidelines in IEColorCurve1.DisplayOptions then
    Guidelines1.State := tssOn
  else
    Guidelines1.State := tssOff;
  if IEColorCurve1.PointShape = iepsCircle then
    CircularPoints1.State := tssOn
  else
    CircularPoints1.State := tssOff;
  if IEColorCurve1.PointSize = 4 then
    SmallFilledPoints1.State := tssOn
  else
    SmallFilledPoints1.State := tssOff;
  GetPresetCurves();
end;

end.
